=== modified file 'NEWS'
--- NEWS	2010-01-12 04:32:16 +0000
+++ NEWS	2010-01-12 08:03:28 +0000
@@ -186,6 +186,9 @@
 Bug Fixes
 *********
 
+* ``bzr annotate`` on another branch with ``-r branch:...`` no longer
+  fails with an ``ObjectNotLocked`` error.  (Andrew Bennetts, #496590)
+
 * ``bzr export dir`` now requests all file content as a record stream,
   rather than requsting the file content one file-at-a-time. This can make
   exporting over the network significantly faster (54min => 9min in one
@@ -204,6 +207,15 @@
 * Give a clearer message if the lockdir disappears after being apparently
   successfully taken.  (Martin Pool, #498378)
 
+* If we fail to open ``~/.bzr.log`` write a clear message to stderr rather
+  than using ``warning()``. The log file is opened before logging is set
+  up, and it leads to very confusing: 'no handlers for "bzr"' messages for
+  users, rather than something nicer.
+  (John Arbash Meinel, Barry Warsaw, #503886)
+
+* Refuse to build with any Pyrex 0.9.4 release, as they have known bugs.
+  (Martin Pool, John Arbash Meinel, #449372)
+
 * The 2a format wasn't properly restarting autopacks when something
   changed underneath it (like another autopack). Now concurrent
   autopackers will properly succeed. (John Arbash Meinel, #495000)

=== modified file 'bzrlib/tests/blackbox/test_annotate.py'
--- bzrlib/tests/blackbox/test_annotate.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_annotate.py	2010-01-07 12:32:37 +0000
@@ -29,6 +29,7 @@
 from bzrlib.branch import Branch
 from bzrlib.config import extract_email_address
 from bzrlib.tests import TestCaseWithTransport
+from bzrlib.urlutils import joinpath
 
 
 class TestAnnotate(TestCaseWithTransport):
@@ -153,14 +154,27 @@
 class TestSimpleAnnotate(TestCaseWithTransport):
     """Annotate tests with no complex setup."""
 
-    def _setup_edited_file(self):
+    def _setup_edited_file(self, relpath='.'):
         """Create a tree with a locally edited file."""
-        tree = self.make_branch_and_tree('.')
-        self.build_tree_contents([('file', 'foo\ngam\n')])
+        tree = self.make_branch_and_tree(relpath)
+        file_relpath = joinpath(relpath, 'file')
+        self.build_tree_contents([(file_relpath, 'foo\ngam\n')])
         tree.add('file')
         tree.commit('add file', committer="test@host", rev_id="rev1")
-        self.build_tree_contents([('file', 'foo\nbar\ngam\n')])
+        self.build_tree_contents([(file_relpath, 'foo\nbar\ngam\n')])
         tree.branch.get_config().set_user_option('email', 'current@host2')
+        return tree
+
+    def test_annotate_cmd_revspec_branch(self):
+        tree = self._setup_edited_file('trunk')
+        tree.branch.create_checkout(self.get_url('work'), lightweight=True)
+        os.chdir('work')
+        out, err = self.run_bzr('annotate file -r branch:../trunk')
+        self.assertEqual('', err)
+        self.assertEqual(
+            '1   test@ho | foo\n'
+            '            | gam\n',
+            out)
 
     def test_annotate_edited_file(self):
         tree = self._setup_edited_file()

=== modified file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	2009-12-16 22:29:31 +0000
+++ bzrlib/tests/test_trace.py	2010-01-12 06:30:41 +0000
@@ -27,6 +27,7 @@
 
 from bzrlib import (
     errors,
+    trace,
     )
 from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.trace import (
@@ -248,6 +249,20 @@
             tmp1.close()
             tmp2.close()
 
+    def test__open_bzr_log_uses_stderr_for_failures(self):
+        # If _open_bzr_log cannot open the file, then we should write the
+        # warning to stderr. Since this is normally happening before logging is
+        # set up.
+        self.addCleanup(setattr, sys, 'stderr', sys.stderr)
+        self.addCleanup(setattr, trace, '_bzr_log_filename',
+                        trace._bzr_log_filename)
+        sys.stderr = StringIO()
+        # Set the log file to something that cannot exist
+        os.environ['BZR_LOG'] = os.getcwd() + '/no-dir/bzr.log'
+        logf = trace._open_bzr_log()
+        self.assertIs(None, logf)
+        self.assertContainsRe(sys.stderr.getvalue(),
+                              'failed to open trace file: .*/no-dir/bzr.log')
 
 class TestVerbosityLevel(TestCase):
 

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2009-12-01 05:35:52 +0000
+++ bzrlib/trace.py	2010-01-12 06:30:41 +0000
@@ -250,7 +250,12 @@
             bzr_log_file.write("bug reports to https://bugs.launchpad.net/bzr/+filebug\n\n")
         return bzr_log_file
     except IOError, e:
-        warning("failed to open trace file: %s" % (e))
+        # If we are failing to open the log, then most likely logging has not
+        # been set up yet. So we just write to stderr rather than using
+        # 'warning()'. If we using warning(), users get the unhelpful 'no
+        # handlers registered for "bzr"' when something goes wrong on the
+        # server. (bug #503886)
+        sys.stderr.write("failed to open trace file: %s\n" % (e,))
     # TODO: What should happen if we fail to open the trace file?  Maybe the
     # objects should be pointed at /dev/null or the equivalent?  Currently
     # returns None which will cause failures later.

=== modified file 'setup.py'
--- setup.py	2009-12-30 23:09:23 +0000
+++ setup.py	2010-01-12 06:30:41 +0000
@@ -282,11 +282,13 @@
     add_pyrex_extension('bzrlib._walkdirs_win32')
     z_lib = 'zdll'
 else:
-    if have_pyrex and pyrex_version == '0.9.4.1':
+    if have_pyrex and pyrex_version.startswith('0.9.4'):
         # Pyrex 0.9.4.1 fails to compile this extension correctly
         # The code it generates re-uses a "local" pointer and
         # calls "PY_DECREF" after having set it to NULL. (It mixes PY_XDECREF
         # which is NULL safe with PY_DECREF which is not.)
+        # <https://bugs.edge.launchpad.net/bzr/+bug/449372>
+        # <https://bugs.edge.launchpad.net/bzr/+bug/276868>
         print 'Cannot build extension "bzrlib._dirstate_helpers_pyx" using'
         print 'your version of pyrex "%s". Please upgrade your pyrex' % (
             pyrex_version,)

